// main.h
const char version[64] = "\nJed Margolin's NRF-905 TX Test   7/8/2024";  // for uart

// GPIO Pins
#define LCD_E 10
#define LCD_RS 11
#define LCD_D4 6
#define LCD_D5 7
#define LCD_D6 8
#define LCD_D7 9

#define PB1 14
#define PB2 13
#define PB3 12
#define SW 15
#define CHANNEL 3    // RF Channel
#define LED 2
#define TXEN 21    // nrf905 Transmit Enable
#define TRX_CE 20  // enable nrf905 chip for transmit/receive
#define AM 28      // Address Match
#define DR 22      // Data Ready
#define CSN905 17  // nrf906 CSN - pin 22
#define BUF_LEN  33

#define UART_TX_PIN 0
#define UART_RX_PIN 1
#define UART_ID uart0
#define BAUD_RATE 115200

#define ADC0_PIN 26

extern struct myswitch switch1;
extern struct myswitch switch2;
extern struct myswitch switch3;

extern void do_switch(unsigned char, struct myswitch *); 
extern unsigned char timer_flag;

// 433.6 MHz
// WCx[0] is the instruction and the address start for nrf905
// TXADRx is the key, Receiver must match the TX key
const unsigned char WC1[11] = { 0x00, 0x70, 0x0C, 0x44, 0x20, 0x20, 0x61, 0x62, 0x63, 0x64, 0xDB };  // 433.6 MHz
const unsigned char TXADR1[5] = { 0x22, 0x61, 0x62, 0x63, 0x64 };  // TXADR[0] is the instruction

// 433.4 MHz
const unsigned char WC2[11] = { 0x00, 0x6E, 0x0C, 0x44, 0x20, 0x20, 0x54, 0x53, 0x52, 0x51, 0xDB };  // 433.4 MHz
const unsigned char TXADR2[5] = { 0x22, 0x54, 0x53, 0x52, 0x51 };  // TXADR[0] is the instruction

// 434.0 MHz
unsigned char WC3[11] = { 0x00, 0x74, 0x0C, 0x44, 0x20, 0x20, 0x54, 0x53, 0x52, 0x51, 0xDB };  // Garage 434.0 MHz
unsigned char TXADR3[5] = { 0x22, 0x54, 0x53, 0x52, 0x51 };   // TXADR[0] is the instruction

unsigned char spi_out_buffer[BUF_LEN];   // out_buffer[0] is the instruction
unsigned char spi_in_buffer[BUF_LEN];    // in_buffer[0] is the status byte

void nrf905_init(void);
void nrf905_tx(void);

struct repeating_timer timer;  // for repeating_timer_callback

// Messages (duplicate in clock.h)
struct messages
{
    unsigned char pos;        // position
    unsigned char numchars;   // number of characters
    unsigned char msg[];      // message
};

uint8_t out_buffer[10];   // first byte is register address
//   Command is: i2c_write_blocking(i2c_default, DSADR, out_buffer, nbytes_out, false);

extern unsigned char nflag;
extern void do_clock(void);
//=======================================================
char nstr[128];
char line[256];

unsigned char mode;   // 0 = RX; 1 = clock
unsigned char data_received, irq_flag, secs_flag;
unsigned int save_count, irq_count; 
unsigned char nflag;
void log_data_ts(void);
void nrf905_transmit(void);
void show_data_lcd(void);

void LED_break(void);
void my_itoa(unsigned int, char *);
void my_itoa2(unsigned int, char *);
void my_hextoa2(unsigned char,char *);

void line_fill_bin(unsigned char,char *, unsigned char, unsigned char);
void line_fill(unsigned char, char *, unsigned char, unsigned char);
void log_time(void);
void log_time2(void);
void log_data(void);
//================================================================
const struct messages tx_msg1 = {0x80, 14, "Jed Margolin's"};
const struct messages tx_msg2 = {0xC2, 7,  "TX Test"};
const struct messages tx_msg3 = {0x83, 10, "TX  7/8/24"};
//=================================================================
// From clock.c
#define SECS   0
#define MINS   1
#define HOURS  2
#define DAY    3
#define DATE   4
#define MONTH  5
#define YEAR   6

extern void i2c_get_time(void);
extern unsigned char time_buffer[];
extern const struct messages msg1, msg2, msg3;

extern void SetChrMode(void);
extern void SetCmdMode(void);
extern void lcd_init(void);
extern void domsg(const struct messages *);
extern void lcd_wr_data(unsigned char);
extern void lcd_wr_instr(unsigned char);
extern void lcd_text(char *);
extern void lcd_byte(char);
extern void lcd_wr_hex(unsigned int, unsigned char);
extern void convert_hex_bcd2(unsigned char, unsigned char *);
extern void lcd_wr_bcd(unsigned int, unsigned char);
extern void lcd_wr_bcd2(unsigned int, unsigned char);
extern void lcd_wr_time(unsigned int, unsigned int);
extern void lcd_wr_hex16(unsigned int, unsigned int);
extern void lcd_wr_bcd4(unsigned int, unsigned int);
extern void clear_lcd(unsigned char);
//================================================================
/*  RPI Pico sizes
Name 	            C++ 	Bytes 	Range

bool 		                    1 	true or false
signed char 	    int8_t 	    1 	-128 to 127
unsigned char 	    uint8_t 	1 	0 to 255
short int 	        int16_t 	2 	-32768 to 32767
unsigned short int 	uint16_t 	2 	0 to 65535
int 	            int32_t 	4 	-2147483648 to 2147483647
unsigned int 	    uint32_t 	4 	0 to 4294967295
long int 	        int32_t 	4 	-2147483648 to 2147483647
unsigned long int 	uint32_t 	4 	0 to 4294967295
long long 	        int64_t 	8 	−9,223,372,036,854,775,808 to 9,223,372,036,854,775,807
unsigned long long 	uint64_t 	8 	0 to 18,446,744,073,709,551,615
float 		                    4 	+/- 3.4e +/- 38 (~7 digits)
double 		                    8 	+/- 1.7e +/- 308 (~15 digits)
long double 		            8 	+/- 1.7e +/- 308 (~15 digits)
wchar_t 		            2 or 4  1 wide character
*/
